@@ -216,6 +216,15 @@ class Arch:
             self.persistent_regs = [r.name for r in self.register_list if r.persistent]
             self.concretize_unique_registers = set(r.vex_offset for r in self.register_list if r.concretize_unique)
             self.artificial_registers = set(r.name for r in self.register_list if r.artificial)
+            self.cpu_flag_register_offsets_and_bitmasks_map = {}
+            self.reg_blacklist = []
+            self.reg_blacklist_offsets = []
+
+            # Artificial registers offsets
+            self.artificial_registers_offsets = []
+            for reg_name in self.artificial_registers:
+                reg = self.get_register_by_name(reg_name)
+                self.artificial_registers_offsets.extend(range(reg.vex_offset, reg.vex_offset + reg.size))
 
             # Register offsets
             try:
@@ -266,12 +275,6 @@ class Arch:
                 if hasattr(self.uc_const, reg_name):
                     self.uc_regs[r] = getattr(self.uc_const, reg_name)
 
-            # Artificial registers offsets
-            self.artificial_registers_offsets = []
-            for reg_name in self.artificial_registers:
-                reg = self.get_register_by_name(reg_name)
-                self.artificial_registers_offsets.extend(range(reg.vex_offset, reg.vex_offset + reg.size))
-
             # VEX register offset to unicorn register ID map
             self.vex_to_unicorn_map = {}
             pc_reg_name = self.get_register_by_name("pc")
@@ -280,7 +283,7 @@ class Arch:
                     continue
 
                 vex_reg = self.get_register_by_name(reg_name)
-                self.vex_to_unicorn_map[vex_reg.vex_offset] = unicorn_reg_id
+                self.vex_to_unicorn_map[vex_reg.vex_offset] = (unicorn_reg_id, vex_reg.size)
 
             # VEX registers used in lieu of flags register
             self.vex_cc_regs = []
@@ -319,6 +322,9 @@ class Arch:
     def __getstate__(self):
         self._cs = None
         self._ks = None
+        if self.vex_archinfo is not None:
+            # clear hwcacheinfo-caches because it may contain cffi.CData
+            self.vex_archinfo['hwcache_info']['caches'] = None
         return self.__dict__
 
     def __setstate__(self, data):
@@ -391,7 +397,7 @@ class Arch:
 
         return fmt_end + fmt_size
 
-    def _get_register_dict(self) ->  Dict[RegisterName, Tuple[RegisterOffset, int]]:
+    def _get_register_dict(self) -> Dict[RegisterName, Tuple[RegisterOffset, int]]:
         res = {}
         for r in self.register_list:
             if r.vex_offset is None:
@@ -706,11 +712,10 @@ class Arch:
     uc_prefix = None
     uc_regs = None
     artificial_registers_offsets = None
-    artificial_registers = set()
+    artificial_registers = None
     cpu_flag_register_offsets_and_bitmasks_map = None
-    reg_blacklist = []
-    reg_blacklist_offsets = []
-    unicorn_flag_register = None
+    reg_blacklist = None
+    reg_blacklist_offsets = None
     vex_to_unicorn_map = None
     vex_cc_regs = None
 
